@@ -9,8 +9,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2013-2016 University of Houston. All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2013-2018 University of Houston. All rights reserved.
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
@@ -35,49 +35,27 @@ int mca_sharedfp_individual_file_open (struct ompi_communicator_t *comm,
 				       const char* filename,
 				       int amode,
 				       struct opal_info_t *info,
-				       mca_io_ompio_file_t *fh)
+				       ompio_file_t *fh)
 {
     int err = 0;
-    int rank;
     char * datafilename;	/*The array size would change as it is based on the current path*/
     char * metadatafilename;	/*The array size would change as it is based on the current path*/
-    mca_io_ompio_file_t * datafilehandle;
-    mca_io_ompio_file_t * metadatafilehandle;
+    ompio_file_t * datafilehandle;
+    ompio_file_t * metadatafilehandle;
     mca_sharedfp_individual_header_record* headnode = NULL;
     struct mca_sharedfp_base_data_t* sh;
-    mca_io_ompio_file_t * shfileHandle;
     size_t len=0;
 
-    /*-------------------------------------------------*/
-    /*Open the same file again without shared file pointer*/
-    /*-------------------------------------------------*/
-    shfileHandle =  (mca_io_ompio_file_t *) malloc ( sizeof(mca_io_ompio_file_t));
-    if ( NULL == shfileHandle ) {
-        opal_output(0, "mca_sharedfp_individual_file_open: unable to allocate memory\n");
-        return OMPI_ERR_OUT_OF_RESOURCE;
-    }
-
-    err = mca_common_ompio_file_open ( comm, filename, amode, info, shfileHandle, false);
-    if ( OMPI_SUCCESS != err )  {
-        opal_output(0, "mca_sharedfp_individual_file_open: Error during file open\n");
-        return err;
-    }
-    shfileHandle->f_fh = fh->f_fh;
-
     sh = (struct mca_sharedfp_base_data_t*) malloc ( sizeof(struct mca_sharedfp_base_data_t));
     if ( NULL == sh ){
         opal_output(0, "mca_sharedfp_individual_file_open: Error, unable to malloc "
 		    "f_sharedfp_ptr struct\n");
-	free ( shfileHandle );
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
-    rank = ompi_comm_rank ( comm );
 
     /*Populate the sh file structure based on the implementation*/
-    sh->sharedfh      = shfileHandle;		/* Shared file pointer*/
     sh->global_offset = 0;			/* Global Offset*/
-    sh->comm          = comm; 			/* Communicator*/
     sh->selected_module_data = NULL;
 
     /* Assign the metadatalinked list to sh->handle */
@@ -97,27 +75,24 @@ int mca_sharedfp_individual_file_open (struct ompi_communicator_t *comm,
     datafilename = (char*)malloc( len );
     if ( NULL == datafilename ) {
         opal_output(0, "mca_sharedfp_individual_file_open: unable to allocate memory\n");
-        free ( shfileHandle );
         free ( sh );
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
-    snprintf(datafilename, len, "%s%s%d",filename,".data.",rank);
+    snprintf(datafilename, len, "%s%s%d",filename,".data.",fh->f_rank);
 
 
-    datafilehandle = (mca_io_ompio_file_t *)malloc(sizeof(mca_io_ompio_file_t));
+    datafilehandle = (ompio_file_t *)malloc(sizeof(ompio_file_t));
     if ( NULL == datafilehandle ) {
         opal_output(0, "mca_sharedfp_individual_file_open: unable to allocate memory\n");
-        free ( shfileHandle );
         free ( sh );
         free ( datafilename );
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
     err = mca_common_ompio_file_open(MPI_COMM_SELF, datafilename,
-                                   MPI_MODE_RDWR | MPI_MODE_CREATE | MPI_MODE_DELETE_ON_CLOSE,
-                                   &(MPI_INFO_NULL->super), datafilehandle, false);
+                                     MPI_MODE_RDWR | MPI_MODE_CREATE | MPI_MODE_DELETE_ON_CLOSE,
+                                     &(MPI_INFO_NULL->super), datafilehandle, false);
     if ( OMPI_SUCCESS != err) {
         opal_output(0, "mca_sharedfp_individual_file_open: Error during datafile file open\n");
-        free (shfileHandle );
         free (sh);
 	free (datafilename);
         free (datafilehandle);
@@ -136,18 +111,16 @@ int mca_sharedfp_individual_file_open (struct ompi_communicator_t *comm,
     /* metadata filename created by appending .metadata.$rank to the original filename*/
     metadatafilename = (char*) malloc ( len );
     if ( NULL == metadatafilename ) {
-        free (shfileHandle );
         free (sh);
 	free (datafilename);
         free (datafilehandle);
         opal_output(0, "mca_sharedfp_individual_file_open: Error during memory allocation\n");
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
-    snprintf ( metadatafilename, len, "%s%s%d", filename, ".metadata.",rank);
+    snprintf ( metadatafilename, len, "%s%s%d", filename, ".metadata.",fh->f_rank);
 
-    metadatafilehandle = (mca_io_ompio_file_t *)malloc(sizeof(mca_io_ompio_file_t));
+    metadatafilehandle = (ompio_file_t *)malloc(sizeof(ompio_file_t));
     if ( NULL == metadatafilehandle ) {
-        free (shfileHandle );
         free (sh);
         free (datafilename);
         free (datafilehandle);
@@ -156,11 +129,10 @@ int mca_sharedfp_individual_file_open (struct ompi_communicator_t *comm,
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
     err = mca_common_ompio_file_open ( MPI_COMM_SELF,metadatafilename,
-                                     MPI_MODE_RDWR | MPI_MODE_CREATE | MPI_MODE_DELETE_ON_CLOSE,
-                                     &(MPI_INFO_NULL->super), metadatafilehandle, false);
+                                       MPI_MODE_RDWR | MPI_MODE_CREATE | MPI_MODE_DELETE_ON_CLOSE,
+                                       &(MPI_INFO_NULL->super), metadatafilehandle, false);
     if ( OMPI_SUCCESS != err) {
         opal_output(0, "mca_sharedfp_individual_file_open: Error during metadatafile file open\n");
-        free (shfileHandle );
         free (sh);
         free (datafilename);
         free (datafilehandle);
@@ -184,23 +156,19 @@ int mca_sharedfp_individual_file_open (struct ompi_communicator_t *comm,
     return err;
 }
 
-int mca_sharedfp_individual_file_close (mca_io_ompio_file_t *fh)
+int mca_sharedfp_individual_file_close (ompio_file_t *fh)
 {
     mca_sharedfp_individual_header_record* headnode = NULL;
     struct mca_sharedfp_base_data_t *sh;
     int err = OMPI_SUCCESS;
 
     if ( NULL == fh->f_sharedfp_data ){
-	if ( mca_sharedfp_individual_verbose ) {
-                opal_output(ompi_sharedfp_base_framework.framework_output,
-                    "sharedfp_inidividual_file_close - shared file pointer structure not initialized\n");
-	}
         return OMPI_SUCCESS;
     }
     sh = fh->f_sharedfp_data;
 
     /* Merge data from individal files to final output file */
-    err = mca_sharedfp_individual_collaborate_data (sh);
+    err = mca_sharedfp_individual_collaborate_data (sh, fh);
 
     headnode = (mca_sharedfp_individual_header_record*)(sh->selected_module_data);
     if (headnode)  {
@@ -229,11 +197,9 @@ int mca_sharedfp_individual_file_close (mca_io_ompio_file_t *fh)
         }
     }
 
-    /* Close the main file opened by this component*/
-    err = mca_common_ompio_file_close(sh->sharedfh);
-
     /*free shared file pointer data struct*/
     free(sh);
+    fh->f_sharedfp_data=NULL;
 
     return err;
 }
