@@ -4,6 +4,8 @@
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
+ * Copyright (c) 2017      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -27,14 +29,14 @@ uint32_t outputFlags = VALIDATE_DATA | CHECK_PACK_UNPACK | RESET_CONVERTORS | QU
 
 static int32_t opal_datatype_create_indexed( int count, const int* pBlockLength, const int* pDisp,
                                              const opal_datatype_t* oldType, opal_datatype_t** newType );
-static int32_t opal_datatype_create_hindexed( int count, const int* pBlockLength, const OPAL_PTRDIFF_TYPE* pDisp,
+static int32_t opal_datatype_create_hindexed( int count, const int* pBlockLength, const ptrdiff_t* pDisp,
                                               const opal_datatype_t* oldType, opal_datatype_t** newType );
 static int32_t opal_datatype_create_struct( int count, const int* pBlockLength,
-                                            const OPAL_PTRDIFF_TYPE* pDisp,
+                                            const ptrdiff_t* pDisp,
                                             opal_datatype_t** pTypes, opal_datatype_t** newType );
 static int32_t opal_datatype_create_vector( int count, int bLength, int stride,
                                             const opal_datatype_t* oldType, opal_datatype_t** newType );
-static int32_t opal_datatype_create_hvector( int count, int bLength, OPAL_PTRDIFF_TYPE stride,
+static int32_t opal_datatype_create_hvector( int count, int bLength, ptrdiff_t stride,
                                              const opal_datatype_t* oldType, opal_datatype_t** newType );
 
 
@@ -136,7 +138,7 @@ opal_datatype_t* test_struct( void )
                                  NULL,
                                  (opal_datatype_t*)&opal_datatype_int1 };
     int lengths[] = { 2, 1, 3 };
-    OPAL_PTRDIFF_TYPE disp[] = { 0, 16, 26 };
+    ptrdiff_t disp[] = { 0, 16, 26 };
     opal_datatype_t* pdt, *pdt1;
 
     printf( "test struct\n" );
@@ -166,7 +168,7 @@ opal_datatype_t* test_struct_char_double( void )
 {
     char_double_t data;
     int lengths[] = {1, 1};
-    OPAL_PTRDIFF_TYPE displ[] = {0, 0};
+    ptrdiff_t displ[] = {0, 0};
     opal_datatype_t *pdt;
     opal_datatype_t* types[] = { (opal_datatype_t*)&opal_datatype_int1,
                                  (opal_datatype_t*)&opal_datatype_float8};
@@ -200,7 +202,7 @@ typedef struct {
 opal_datatype_t* create_strange_dt( void )
 {
     sdata_intern v[2];
-    OPAL_PTRDIFF_TYPE displ[3];
+    ptrdiff_t displ[3];
     opal_datatype_t *pdt, *pdt1;
 
     opal_datatype_create_contiguous(0, &opal_datatype_empty, &pdt1);
@@ -280,7 +282,7 @@ static int32_t opal_datatype_create_indexed( int count, const int* pBlockLength,
 {
     opal_datatype_t* pdt;
     int i, dLength, endat, disp;
-    OPAL_PTRDIFF_TYPE extent;
+    ptrdiff_t extent;
 
     if( 0 == count ) {
         *newType = opal_datatype_create( 0 );
@@ -317,12 +319,12 @@ static int32_t opal_datatype_create_indexed( int count, const int* pBlockLength,
     return OPAL_SUCCESS;
 }
 
-static int32_t opal_datatype_create_hindexed( int count, const int* pBlockLength, const OPAL_PTRDIFF_TYPE* pDisp,
+static int32_t opal_datatype_create_hindexed( int count, const int* pBlockLength, const ptrdiff_t* pDisp,
                                               const opal_datatype_t* oldType, opal_datatype_t** newType )
 {
     opal_datatype_t* pdt;
     int i, dLength;
-    OPAL_PTRDIFF_TYPE extent, disp, endat;
+    ptrdiff_t extent, disp, endat;
 
     if( 0 == count ) {
         *newType = opal_datatype_create( 0 );
@@ -360,11 +362,11 @@ static int32_t opal_datatype_create_hindexed( int count, const int* pBlockLength
 }
 
 
-static int32_t opal_datatype_create_struct( int count, const int* pBlockLength, const OPAL_PTRDIFF_TYPE* pDisp,
+static int32_t opal_datatype_create_struct( int count, const int* pBlockLength, const ptrdiff_t* pDisp,
                                             opal_datatype_t** pTypes, opal_datatype_t** newType )
 {
     int i;
-    OPAL_PTRDIFF_TYPE disp = 0, endto, lastExtent, lastDisp;
+    ptrdiff_t disp = 0, endto, lastExtent, lastDisp;
     int lastBlock;
     opal_datatype_t *pdt, *lastType;
 
@@ -433,7 +435,7 @@ static int32_t opal_datatype_create_vector( int count, int bLength, int stride,
                                             const opal_datatype_t* oldType, opal_datatype_t** newType )
 {
     opal_datatype_t *pTempData, *pData;
-    OPAL_PTRDIFF_TYPE extent = oldType->ub - oldType->lb;
+    ptrdiff_t extent = oldType->ub - oldType->lb;
 
 
     if( 0 == count ) {
@@ -461,11 +463,11 @@ static int32_t opal_datatype_create_vector( int count, int bLength, int stride,
 }
 
 
-static int32_t opal_datatype_create_hvector( int count, int bLength, OPAL_PTRDIFF_TYPE stride,
+static int32_t opal_datatype_create_hvector( int count, int bLength, ptrdiff_t stride,
                                              const opal_datatype_t* oldType, opal_datatype_t** newType )
 {
     opal_datatype_t *pTempData, *pData;
-    OPAL_PTRDIFF_TYPE extent = oldType->ub - oldType->lb;
+    ptrdiff_t extent = oldType->ub - oldType->lb;
 
     if( 0 == count ) {
         *newType = opal_datatype_create( 0 );
@@ -609,7 +611,7 @@ opal_datatype_t* test_contiguous( void )
 {
     opal_datatype_t *pdt, *pdt1, *pdt2;
 
-    printf( "test contiguous (alignement)\n" );
+    printf( "test contiguous (alignment)\n" );
     opal_datatype_create_contiguous(0, &opal_datatype_empty, &pdt1);
     opal_datatype_add( pdt1, &opal_datatype_float8, 1, 0, -1 );
     if( outputFlags & DUMP_DATA_AFTER_COMMIT ) {
@@ -633,236 +635,3 @@ opal_datatype_t* test_contiguous( void )
 }
 
 
-
-int mpich_typeub( void )
-{
-   int errs = 0;
-   OPAL_PTRDIFF_TYPE extent, lb, extent1, extent2, extent3;
-   OPAL_PTRDIFF_TYPE displ[2];
-   int blens[2];
-   opal_datatype_t *type1, *type2, *type3, *types[2];
-
-   opal_datatype_create_vector( 2, 1, 4, &opal_datatype_int4, &type1 );
-   opal_datatype_commit( type1 );
-   opal_datatype_get_extent( type1, &lb, &extent );
-   extent1 = 5 * sizeof(int);
-   if (extent != extent1) {
-      printf("EXTENT 1 %ld != %ld\n", (long)extent, (long)extent1);
-      errs++;
-      printf("extent(type1)=%ld\n",(long)extent);
-   }
-
-   blens[0] = 1;
-   blens[1] = 1;
-   displ[0] = 0;
-   displ[1] = sizeof(int)*4;
-   types[0] = type1;
-   types[1] = (opal_datatype_t*)&opal_datatype_ub;
-   extent2  = displ[1];
-
-   /*    using MPI_UB and Type_struct, monkey with the extent, making it 16
-    */
-   opal_datatype_create_struct( 2, blens, displ, types, &type2 );
-   opal_datatype_commit( type2 );
-   opal_datatype_get_extent( type2, &lb, &extent );
-   if (extent != extent2) {
-      printf("EXTENT 2 %ld != %ld\n", (long)extent, (long)extent2);
-      errs++;
-      printf("extent(type2)=%ld\n",(long)extent);
-   }
-
-   /*    monkey with the extent again, making it 4
-    *     ===> MPICH gives 4
-    *     ===> MPIF gives 16, the old extent
-    */
-   displ[1] = sizeof(int);
-   types[0] = type2;
-   types[1] = (opal_datatype_t*)&opal_datatype_ub;
-   extent3  = extent2;
-
-   opal_datatype_create_struct( 2, blens, displ, types, &type3 );
-   opal_datatype_commit( type3 );
-
-   opal_datatype_get_extent( type3, &lb, &extent );
-   if (extent != extent3) {
-      printf("EXTENT 3 %ld != %ld\n", (long)extent, (long)extent3);
-      errs++;
-      printf("extent(type3)=%ld\n",(long)extent);
-   }
-
-   OBJ_RELEASE( type1 ); /*assert( type1 == NULL );*/
-   OBJ_RELEASE( type2 ); /*assert( type2 == NULL );*/
-   OBJ_RELEASE( type3 ); assert( type3 == NULL );
-   return errs;
-}
-
-int mpich_typeub2( void )
-{
-   int blocklen[3], err = 0;
-   size_t sz1, sz2, sz3;
-   OPAL_PTRDIFF_TYPE disp[3], lb, ub, ex1, ex2, ex3;
-   opal_datatype_t *types[3], *dt1, *dt2, *dt3;
-
-   blocklen[0] = 1;
-   blocklen[1] = 1;
-   blocklen[2] = 1;
-   disp[0] = -3;
-   disp[1] = 0;
-   disp[2] = 6;
-   types[0] = (opal_datatype_t*)&opal_datatype_lb;
-   types[1] = (opal_datatype_t*)&opal_datatype_int4;
-   types[2] = (opal_datatype_t*)&opal_datatype_ub;
-
-   opal_datatype_create_struct(3, blocklen, disp, types, &dt1);
-   opal_datatype_commit( dt1 );
-
-   opal_datatype_type_lb(dt1, &lb);          opal_datatype_type_ub(dt1, &ub);
-   opal_datatype_type_extent(dt1,&ex1);      opal_datatype_type_size(dt1,&sz1);
-
-   /* Values should be lb = -3, ub = 6 extent 9; size depends on implementation */
-   if (lb != -3 || ub != 6 || ex1 != 9) {
-      printf("Example 3.26 type1 lb %d ub %d extent %d size %d\n", (int)lb, (int)ub, (int)ex1, (int)sz1);
-      err++;
-   }
-   else
-      printf("Example 3.26 type1 correct\n" );
-
-   opal_datatype_create_contiguous(2, dt1, &dt2);
-   opal_datatype_type_lb(dt2, &lb);          opal_datatype_type_ub(dt2, &ub);
-   opal_datatype_type_extent(dt2,&ex2);      opal_datatype_type_size(dt2,&sz2);
-   /* Values should be lb = -3, ub = 15, extent = 18, size depends on implementation */
-   if (lb != -3 || ub != 15 || ex2 != 18) {
-      printf("Example 3.26 type2 lb %d ub %d extent %d size %d\n", (int)-3, (int)15, (int)18, 8);
-      printf("Example 3.26 type2 lb %d ub %d extent %d size %d\n", (int)lb, (int)ub, (int)ex2, (int)sz2);
-      err++;
-   }
-   else
-      printf("Example 3.26 type1 correct\n" );
-   OBJ_RELEASE( dt2 ); assert( dt2 == NULL );
-   opal_datatype_create_contiguous(2,dt1,&dt2);
-   opal_datatype_type_lb(dt2, &lb);          opal_datatype_type_ub(dt2, &ub);
-   opal_datatype_type_extent(dt2,&ex2);      opal_datatype_type_size(dt2,&sz2);
-   /* Values should be lb = -3, ub = 15, extent = 18, size depends on implementation */
-   if (lb != -3 || ub != 15 || ex2 != 18) {
-      printf("Example 3.26 type2 lb %d ub %d extent %d size %d\n", (int)-3, (int)15, (int)18, 8);
-      printf("Example 3.26 type2 lb %d ub %d extent %d size %d\n", (int)lb, (int)ub, (int)ex2, (int)sz2);
-      err++;
-   }
-   else
-      printf( "Example 3.26 type2 correct\n" );
-
-   types[0]=dt1;               types[1]=dt1;
-   blocklen[0]=1;              blocklen[1]=1;
-   disp[0]=0;                  disp[1]=ex1;
-
-   opal_datatype_create_struct(2, blocklen, disp, types, &dt3);
-   opal_datatype_commit( dt3 );
-
-   opal_datatype_type_lb(dt3, &lb);          opal_datatype_type_ub(dt3, &ub);
-   opal_datatype_type_extent(dt3,&ex3);      opal_datatype_type_size(dt3,&sz3);
-   /* Another way to express type2 */
-   if (lb != -3 || ub != 15 || ex3 != 18) {
-      printf("type3 lb %d ub %d extent %d size %d\n", (int)-3, (int)15, (int)18, 8);
-      printf("type3 lb %d ub %d extent %d size %d\n", (int)lb, (int)ub, (int)ex3, (int)sz2);
-      err++;
-   }
-   else
-      printf( "type3 correct\n" );
-
-   OBJ_RELEASE( dt1 ); /*assert( dt1 == NULL );*/
-   OBJ_RELEASE( dt2 ); /*assert( dt2 == NULL );*/
-   OBJ_RELEASE( dt3 ); assert( dt3 == NULL );
-   return err;
-}
-
-int mpich_typeub3( void )
-{
-   int blocklen[3], err = 0, idisp[3];
-   size_t sz;
-   OPAL_PTRDIFF_TYPE disp[3], lb, ub, ex;
-   opal_datatype_t *types[3], *dt1, *dt2, *dt3, *dt4, *dt5;
-
-   /* Create a datatype with explicit LB and UB */
-   blocklen[0] = 1;
-   blocklen[1] = 1;
-   blocklen[2] = 1;
-   disp[0] = -3;
-   disp[1] = 0;
-   disp[2] = 6;
-   types[0] = (opal_datatype_t*)&opal_datatype_lb;
-   types[1] = (opal_datatype_t*)&opal_datatype_int4;
-   types[2] = (opal_datatype_t*)&opal_datatype_ub;
-
-   /* Generate samples for contiguous, hindexed, hvector, indexed, and vector (struct and contiguous tested in typeub2) */
-   opal_datatype_create_struct(3, blocklen, disp, types, &dt1);
-   opal_datatype_commit( dt1 );
-
-   /* This type is the same as in typeub2, and is tested there */
-   types[0]=dt1;               types[1]=dt1;
-   blocklen[0]=1;              blocklen[1]=1;
-   disp[0]=-4;                 disp[1]=7;
-   idisp[0]=-4;                idisp[1]=7;
-
-   opal_datatype_create_hindexed( 2, blocklen, disp, dt1, &dt2 );
-   opal_datatype_commit( dt2 );
-
-   opal_datatype_type_lb( dt2, &lb );       opal_datatype_type_ub( dt2, &ub );
-   opal_datatype_type_extent( dt2, &ex );   opal_datatype_type_size( dt2, &sz );
-
-   if (lb != -7 || ub != 13 || ex != 20) {
-      printf("hindexed lb %d ub %d extent %d size %d\n", (int)-7, (int)13, (int)20, (int)sz);
-      printf("hindexed lb %d ub %d extent %d size %d\n", (int)lb, (int)ub, (int)ex, (int)sz);
-      err++;
-   }
-   else
-      printf( "hindexed ok\n" );
-
-   opal_datatype_create_indexed( 2, blocklen, idisp, dt1, &dt3 );
-   opal_datatype_commit( dt3 );
-
-   opal_datatype_type_lb( dt3, &lb );       opal_datatype_type_ub( dt3, &ub );
-   opal_datatype_type_extent( dt3, &ex );   opal_datatype_type_size( dt3, &sz );
-
-   if (lb != -39 || ub != 69 || ex != 108) {
-      printf("indexed lb %d ub %d extent %d size %d\n", (int)-39, (int)69, (int)108, (int)sz);
-      printf("indexed lb %d ub %d extent %d size %d\n", (int)lb, (int)ub, (int)ex, (int)sz);
-      err++;
-   }
-   else
-      printf( "indexed ok\n" );
-
-   opal_datatype_create_hvector( 2, 1, 14, dt1, &dt4 );
-   opal_datatype_commit( dt4 );
-
-   opal_datatype_type_lb( dt4, &lb );       opal_datatype_type_ub( dt4, &ub );
-   opal_datatype_type_extent( dt4, &ex );   opal_datatype_type_size( dt4, &sz );
-
-   if (lb != -3 || ub != 20 || ex != 23) {
-      printf("hvector lb %d ub %d extent %d size %d\n", (int)-3, (int)20, (int)23, (int)sz);
-      printf("hvector lb %d ub %d extent %d size %d\n", (int)lb, (int)ub, (int)ex, (int)sz);
-      err++;
-   }
-   else
-      printf( "hvector ok\n" );
-
-   opal_datatype_create_vector( 2, 1, 14, dt1, &dt5 );
-   opal_datatype_commit( dt5 );
-
-   opal_datatype_type_lb( dt5, &lb );       opal_datatype_type_ub( dt5, &ub );
-   opal_datatype_type_extent( dt5, &ex );   opal_datatype_type_size( dt5, &sz );
-
-   if (lb != -3 || ub != 132 || ex != 135) {
-      printf("vector lb %d ub %d extent %d size %d\n", (int)-3, (int)132, (int)135, (int)sz);
-      printf("vector lb %d ub %d extent %d size %d\n", (int)lb, (int)ub, (int)ex, (int)sz);
-      err++;
-   }
-   else
-      printf( "vector ok\n" );
-
-   OBJ_RELEASE( dt1 ); /*assert( dt1 == NULL );*/
-   OBJ_RELEASE( dt2 ); /*assert( dt2 == NULL );*/
-   OBJ_RELEASE( dt3 ); /*assert( dt3 == NULL );*/
-   OBJ_RELEASE( dt4 ); /*assert( dt4 == NULL );*/
-   OBJ_RELEASE( dt5 ); assert( dt5 == NULL );
-   return err;
-}
